@@ -1,7 +1,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -130,14 +130,15 @@ def get_type_hints(thing):
             # comprehensive type information from get_type_hints
             # See https://github.com/HypothesisWorks/hypothesis/pull/2580
             # for more details.
-            spec = inspect.getfullargspec(thing)
-            hints.update(
-                {
-                    k: v
-                    for k, v in spec.annotations.items()
-                    if k in (spec.args + spec.kwonlyargs) and isinstance(v, type)
-                }
-            )
+            from hypothesis.strategies._internal.types import is_a_type
+
+            vkinds = (inspect.Parameter.VAR_POSITIONAL, inspect.Parameter.VAR_KEYWORD)
+            for p in inspect.signature(thing).parameters.values():
+                if p.kind not in vkinds and is_a_type(p.annotation):
+                    if p.default is None:
+                        hints[p.name] = typing.Optional[p.annotation]
+                    else:
+                        hints[p.name] = p.annotation
     except (AttributeError, TypeError, NameError):
         pass
 
